/*
 * actionlist.cpp - the customizeable action list
 * Copyright (C) 2004  Michail Pishchagin
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "actionlist.h"

#include <qdict.h>

#include "iconaction.h"

//----------------------------------------------------------------------------
// ActionList
//----------------------------------------------------------------------------

class ActionList::Private
{
public:
	Private() { }

	QString name;
	int id;
	QStringList sortedActions;
	QDict<IconAction> actions;
};

ActionList::ActionList( QString name, int id )
{
	d = new Private();
	d->actions.setAutoDelete( true );

	d->name = name;
	d->id   = id;
}

ActionList::~ActionList()
{
	delete d;
}

QString ActionList::name() const
{
	return d->name;
}

int ActionList::id() const
{
	return d->id;
}

IconAction *ActionList::action( QString name ) const
{
	return d->actions[name];
}

QStringList ActionList::actions() const
{
	return d->sortedActions;
}

void ActionList::addAction( QString name, IconAction *action )
{
	d->sortedActions << name;

	if ( action ) {
		action->setName( name.latin1() );
		d->actions.insert( name, action );
	}
}

void ActionList::clear()
{
	d->actions.clear();
}

//----------------------------------------------------------------------------
// MetaActionList
//----------------------------------------------------------------------------

class MetaActionList::Private
{
public:
	Private() { }

	QPtrList<ActionList> lists;
};

MetaActionList::MetaActionList()
{
	d = new Private();
	d->lists.setAutoDelete( true );
}

MetaActionList::~MetaActionList()
{
	delete d;
}

ActionList *MetaActionList::actionList( QString name ) const
{
	QPtrList<ActionList>::Iterator it = d->lists.begin();
	for ( ; it != d->lists.end(); ++it ) {
		if ( (*it)->name() == name )
			return *it;
	}

	return 0;
}

QPtrList<ActionList> MetaActionList::actionLists( int id ) const
{
	QPtrList<ActionList> list;

	for ( int i = 0; i < 32; i++ ) {
		if ( !(id & ( 1 << i )) )
			continue;

		QPtrList<ActionList>::Iterator it = d->lists.begin();
		for ( ; it != d->lists.end(); ++it ) {
			ActionList *l = *it;

			if ( l->id() & ( 1 << i ) )
				list.append( l );
		}
	}

	return list;
}

QStringList MetaActionList::actionLists() const
{
	QStringList names;

	QPtrList<ActionList>::Iterator it = d->lists.begin();
	for ( ; it != d->lists.end(); ++it )
		names << (*it)->name();

	return names;
}

void MetaActionList::addList( ActionList *list )
{
	if ( list )
		d->lists.append( list );
}

void MetaActionList::clear()
{
	QPtrList<ActionList>::Iterator it = d->lists.begin();
	for ( ; it != d->lists.end(); ++it )
		( *it )->clear();
}
